/* DataSorting.java

	Purpose:
		
	Description:
		
	History:
		5:34 PM 2022/9/8, Created by jumperchen

Copyright (C) 2022 Potix Corporation. All Rights Reserved.
*/
package org.zkoss.chart;

import org.zkoss.chart.util.DynamicalAttribute;

/**
 * Options for the series data sorting.
 * @author jumperchen
 * @since 10.2.1.0
 */
public class DataSorting extends Optionable {

	private enum Attrs implements PlotAttribute, DynamicalAttribute {
		enabled,
		matchByName,
		sortKey
	}

	/**
	 * Enable or disable data sorting for the series. Use {@code xAxis.reversed} to
	 * change the sorting order.
	 * <p>Default: false.
	 */
	public boolean isEnabled() {
		return getAttr(Attrs.enabled, false).asBoolean();
	}

	/**
	 * Enable or disable data sorting for the series. Use {@code xAxis.reversed} to
	 * change the sorting order.
	 * <p>Default: false.
	 */
	public void setEnabled(boolean enabled) {
		setAttr(Attrs.enabled, enabled);
	}

	/**
	 * Whether to allow matching points by name in an update. If this option is
	 * disabled, points will be matched by order.
	 * <p>Default: false.
	 */
	public boolean isMatchByName() {
		return getAttr(Attrs.matchByName, false).asBoolean();
	}

	/**
	 * Whether to allow matching points by name in an update. If this option is
	 * disabled, points will be matched by order.
	 * <p>Default: false.
	 */
	public void setMatchByName(boolean matchByName) {
		setAttr(Attrs.matchByName, matchByName);
	}

	/**
	 * Determines what data value should be used to sort by.
	 *
	 * <p>Defaults to {@code "y"}.
	 */
	public String getSortKey() {
		return getAttr(Attrs.sortKey, "y").asString();
	}

	/**
	 * Determines what data value should be used to sort by.
	 *
	 * <p>Defaults to {@code "y"}.
	 */
	public void setSortKey(String sortKey) {
		setAttr(Attrs.sortKey, sortKey);
	}
}