/* LangSeriesTypeDescriptions.java

	Purpose:
		
	Description:
		
	History:
		12:11 PM 2022/9/20, Created by jumperchen

Copyright (C) 2022 Potix Corporation. All Rights Reserved.
*/
package org.zkoss.chart;

import org.zkoss.chart.util.DynamicalAttribute;

/**
 * Descriptions of lesser known series types. The relevant description is added
 * to the screen reader information region when these series types are used.
 * @author jumperchen
 * @since 10.2.1.0
 */
public class LangSeriesTypeDescriptions extends Optionable {
	private enum Attrs implements PlotAttribute, DynamicalAttribute {
		arearange,
		areasplinerange,
		boxplot,
		bubble,
		columnrange,
		errorbar,
		funnel,
		pyramid,
		waterfall
	}

	/**
	 * <p>Default: {@code "Arearange charts are line charts displaying a range between a lower and higher value for each point."}
	 */
	public String getArearange() {
		return getAttr(Attrs.arearange,
				"Arearange charts are line charts displaying a range between a lower and higher value for each point.").asString();
	}

	public void setArearange(String arearange) {
		setAttr(Attrs.arearange, arearange);
	}

	/**
	 * <p>Default: {@code "These charts are line charts displaying a range between a lower and higher value for each point."}
	 */
	public String getAreasplinerange() {
		return getAttr(Attrs.areasplinerange,
				"These charts are line charts displaying a range between a lower and higher value for each point.").asString();
	}

	public void setAreasplinerange(String areasplinerange) {
		setAttr(Attrs.areasplinerange, areasplinerange);
	}

	/**
	 * <p>Default: {@code "Box plot charts are typically used to display groups of statistical data. Each data point in the chart can have up to 5 values: minimum, lower quartile, median, upper quartile, and maximum."}
	 */
	public String getBoxplot() {
		return getAttr(Attrs.boxplot, "Box plot charts are typically used to display groups of statistical data. Each data point in the chart can have up to 5 values: minimum, lower quartile, median, upper quartile, and maximum.").asString();
	}

	public void setBoxplot(String boxplot) {
		setAttr(Attrs.boxplot, boxplot);
	}

	/**
	 * <p>Default: {@code "Bubble charts are scatter charts where each data point also has a size value."}
	 */
	public String getBubble() {
		return getAttr(Attrs.bubble, "Bubble charts are scatter charts where each data point also has a size value.").asString();
	}

	public void setBubble(String bubble) {
		setAttr(Attrs.bubble, bubble);
	}

	/**
	 * <p>Default: {@code "Columnrange charts are column charts displaying a range between a lower and higher value for each point."}
	 */
	public String getColumnrange() {
		return getAttr(Attrs.columnrange,
				"Columnrange charts are column charts displaying a range between a lower and higher value for each point.").asString();
	}

	public void setColumnrange(String columnrange) {
		setAttr(Attrs.columnrange, columnrange);
	}

	/**
	 * <p>Default: {@code "Errorbar series are used to display the variability of the data."}
	 */
	public String getErrorbar() {
		return getAttr(Attrs.errorbar, "Errorbar series are used to display the variability of the data.").asString();
	}

	public void setErrorbar(String errorbar) {
		setAttr(Attrs.errorbar, errorbar);
	}

	/**
	 * <p>Default: {@code "Funnel charts are used to display reduction of data in stages."}
	 */
	public String getFunnel() {
		return getAttr(Attrs.funnel, "Funnel charts are used to display reduction of data in stages.").asString();
	}

	public void setFunnel(String funnel) {
		setAttr(Attrs.funnel, funnel);
	}

	/**
	 * <p>Default: {@code "Pyramid charts consist of a single pyramid with item heights corresponding to each point value."}
	 */
	public String getPyramid() {
		return getAttr(Attrs.pyramid, "Pyramid charts consist of a single pyramid with item heights corresponding to each point value.").asString();
	}

	public void setPyramid(String pyramid) {
		setAttr(Attrs.pyramid, pyramid);
	}

	/**
	 * <p>Default: {@code "A waterfall chart is a column chart where each column contributes towards a total end value."}
	 */
	public String getWaterfall() {
		return getAttr(Attrs.waterfall, "A waterfall chart is a column chart where each column contributes towards a total end value.").asString();
	}

	public void setWaterfall(String waterfall) {
		setAttr(Attrs.waterfall, waterfall);
	}
}
