/* OnPoint.java

	Purpose:
		
	Description:
		
	History:
		6:04 PM 2022/9/8, Created by jumperchen

Copyright (C) 2022 Potix Corporation. All Rights Reserved.
*/
package org.zkoss.chart;

import org.zkoss.chart.util.DynamicalAttribute;

/**
 * Options for the Series on point feature.
 * Only {@code pie} and {@code sunburst} series are supported at this moment.
 * @author jumperchen
 * @since 10.2.1.0
 */
public class OnPoint extends Optionable {
	private enum Attrs implements PlotAttribute, DynamicalAttribute {
		connectorOptions,
		id,
		position
	}

	/**
	 * Options for the connector in the Series on point feature.
	 * <p>Default: an instance of ConnectorOptions.
	 * @see ConnectorOptions
	 */
	public ConnectorOptions getConnectorOptions() {
		return (ConnectorOptions) getAttr(Attrs.connectorOptions, ConnectorOptions.class).asValue();
	}

	/**
	 * Options for the connector in the Series on point feature.
	 * <p>Default: an instance of ConnectorOptions.
	 * @see ConnectorOptions
	 */
	public void setConnectorOptions(ConnectorOptions connectorOptions) {
		setAttr(Attrs.connectorOptions, connectorOptions);
	}

	/**
	 * The id of the point that we connect the series to. Only points with a given
	 * {@code plotX} and {@code plotY} values and map points are valid.
	 * <p>Default: null.
	 */
	public String getId() {
		return getAttr(Attrs.id, null).asString();
	}

	/**
	 * The id of the point that we connect the series to. Only points with a given
	 * {@code plotX} and {@code plotY} values and map points are valid.
	 * <p>Default: null.
	 */
	public void setId(String id) {
		setAttr(Attrs.id, id);
	}

	/**
	 * Options allowing to set a position and an offset of the series in the Series
	 * on point feature.
	 * <p>Default: an instance of Position.
	 * @see Position
	 */
	public Position getPosition() {
		return (Position) getAttr(Attrs.position, Position.class).asValue();
	}

	/**
	 * Options allowing to set a position and an offset of the series in the Series
	 * on point feature.
	 * <p>Default: an instance of Position.
	 * @see Position
	 */
	public void setPosition(Position position) {
		setAttr(Attrs.position, position);
	}
}
