/* PDFFont.java

	Purpose:
		
	Description:
		
	History:
		12:25 PM 2022/9/22, Created by jumperchen

Copyright (C) 2022 Potix Corporation. All Rights Reserved.
*/
package org.zkoss.chart;

import org.zkoss.chart.util.DynamicalAttribute;

/**
 * Settings for a custom font for the exported PDF. This is used for languages containing
 * non-ASCII characters, like Chinese, Russian, Japanese etc.
 * <p>
 * As described in the <a href="https://github.com/parallax/jsPDF#use-of-unicode-characters--utf-8">jsPDF</a>
 * docs, the 14 standard fonts in PDF are limited to the ASCII-codepage. Therefore,
 * in order to support other text in the exported PDF, one or more TTF font files
 * have to be passed on to the exporting module.
 *
 * <p>
 * See more in <a href="https://www.highcharts.com/docs/export-module/client-side-export">the docs</a>.
 * @author jumperchen
 * @since 10.2.1.0
 */
public class PDFFont extends Optionable {
	private enum Attrs implements PlotAttribute, DynamicalAttribute {
		bold,
		bolditalic,
		italic,
		normal
	}

	/**
	 * Returns the TTF font file for bold text.
	 * <p>Default: {@code null}
	 */
	public String getBold() {
		return getAttr(Attrs.bold, null).asString();
	}

	/**
	 * Sets the TTF font file for bold text.
	 */
	public void setBold(String bold) {
		setAttr(Attrs.bold, bold);
	}

	/**
	 * Returns the TTF font file for bold and italic text.
	 * <p>Default: {@code null}
	 */
	public String getBolditalic() {
		return getAttr(Attrs.bolditalic, null).asString();
	}

	/**
	 * Sets the TTF font file for bold and italic text.
	 */
	public void setBolditalic(String bolditalic) {
		setAttr(Attrs.bolditalic, bolditalic);
	}

	/**
	 * Returns the TTF font file for italic text.
	 * <p>Default: {@code null}
	 */
	public String getItalic() {
		return getAttr(Attrs.italic, null).asString();
	}

	/**
	 * Sets the TTF font file for italic text.
	 */
	public void setItalic(String italic) {
		setAttr(Attrs.italic, italic);
	}

	/**
	 * Returns the TTF font file for normal {@code font-style}. If font variations like
	 * {@code bold} or {@code italic} are not defined, the {@code normal" font will be used for those too.
	 * <p> Default: {@code null}
	 */
	public String getNormal() {
		return getAttr(Attrs.normal, null).asString();
	}

	/**
	 * Sets the TTF font file for normal {@code font-style}. If font variations like
	 * {@code bold} or {@code italic} are not defined, the {@code normal" font will be used for those too.
	 */
	public void setNormal(String normal) {
		setAttr(Attrs.normal, normal);
	}
}
