/* SeriesAccessibility.java

	Purpose:
		
	Description:
		
	History:
		9:52 AM 2022/9/20, Created by jumperchen

Copyright (C) 2022 Potix Corporation. All Rights Reserved.
*/
package org.zkoss.chart;

import org.zkoss.chart.util.DynamicalAttribute;

/**
 * Accessibility options for a series.
 *
 * <p>
 * Requires the accessibility module
 * to be loaded. For example,
 * <pre><code>
 * Library.setProperty("org.zkoss.chart.modules.accessibility", "true");
 * </code></pre>
 *
 * @author jumperchen
 * @since 10.2.1.0
 */
public class SeriesAccessibility extends Optionable {
	private enum Attrs implements PlotAttribute, DynamicalAttribute {
		description,
		enabled,
		exposeAsGroupOnly,
		keyboardNavigation,
		point
	}

	/**
	 * Returns a description of the series, announced to screen readers.
	 * <p>Default: {@code null}
	 */
	public String getDescription() {
		return getAttr(Attrs.description, null).asString();
	}

	/**
	 * Sets a description of the series, announced to screen readers.
	 */
	public void setDescription(String description) {
		setAttr(Attrs.description, description);
	}

	/**
	 * Returns whether to enable/disable accessibility functionality for a specific series.
	 * <p> Default: {@code null}
	 */
	public Boolean isEnabled() {
		return getAttr(Attrs.enabled, null).asBoolean();
	}

	/**
	 * Sets whether to enable/disable accessibility functionality for a specific series.
	 */
	public void setEnabled(boolean enabled) {
		setAttr(Attrs.enabled, enabled);
	}

	/**
	 * Returns to expose only the series element to screen readers, not its points.
	 * <p> Default: {@code null}
	 */
	public Boolean isExposeAsGroupOnly() {
		return getAttr(Attrs.exposeAsGroupOnly, null).asBoolean();
	}

	/**
	 * Sets to expose only the series element to screen readers, not its points.
	 */
	public void setExposeAsGroupOnly(boolean exposeAsGroupOnly) {
		setAttr(Attrs.exposeAsGroupOnly, exposeAsGroupOnly);
	}

	/**
	 * Returns the keyboard navigation for a series, if null, create a new one.
	 * <p> Default: an instance of {@link SeriesNavigation}
	 */
	public SeriesKeyboardNavigation getKeyboardNavigation() {
		return (SeriesKeyboardNavigation) getAttr(
				Attrs.keyboardNavigation, SeriesKeyboardNavigation.class).asValue();
	}

	/**
	 * Sets the keyboard navigation for a series.
	 */
	public void setKeyboardNavigation(
			SeriesKeyboardNavigation keyboardNavigation) {
		setAttr(Attrs.keyboardNavigation, keyboardNavigation);
	}

	/**
	 * Returns the point accessibility options for a series, if null, create a
	 * new one.
	 * <p>Default: an instance  of {@link AccessibilityPoint}
	 */
	public AccessibilityPoint getPoint() {
		return (AccessibilityPoint) getAttr(Attrs.point,
				AccessibilityPoint.class).asValue();
	}

	/**
	 * Sets the point accessibility options for a series.
	 */
	public void setPoint(AccessibilityPoint point) {
		setAttr(Attrs.point, point);
	}
}
