package org.zkoss.chart.model;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.zkoss.zul.event.ChartDataEvent;

public class DefaultBoxPlotModel extends AbstractChartsModel implements BoxPlotModel {

	private static final long serialVersionUID = -2981703469210922704L;
	protected Map<Comparable<?>, List<BoxplotTuple>> _seriesMap = new HashMap<Comparable<?>, List<BoxplotTuple>>();
	protected List<Comparable<?>> _seriesList = new ArrayList<Comparable<?>>();
	
	public Comparable<?> getSeries(int index) {
		return _seriesList.get(index);
	}

	public Collection<Comparable<?>> getSeries() {
		return _seriesList;
	}

	public int getDataCount(Comparable<?> series) {
		final List<BoxplotTuple> boxplotTuples = _seriesMap.get(series);
		return boxplotTuples != null ? boxplotTuples.size() : 0;
	}

	public String getName(Comparable<?> series, int index) {
		final List<BoxplotTuple> boxplotTuples = _seriesMap.get(series);
		if (boxplotTuples != null) {
			return boxplotTuples.get(index).getName();
		}
		return null;
	}

	public Number getX(Comparable<?> series, int index) {
		final List<BoxplotTuple> boxplotTuples = _seriesMap.get(series);
		if (boxplotTuples != null) {
			return boxplotTuples.get(index).getX();
		}
		return null;
	}

	public Number getLow(Comparable<?> series, int index) {
		final List<BoxplotTuple> boxplotTuples = _seriesMap.get(series);
		if (boxplotTuples != null) {
			return boxplotTuples.get(index).getLow();
		}
		return null;
	}

	public Number getQ1(Comparable<?> series, int index) {
		final List<BoxplotTuple> boxplotTuples = _seriesMap.get(series);
		if (boxplotTuples != null) {
			return boxplotTuples.get(index).getQ1();
		}
		return null;
	}

	public Number getMedian(Comparable<?> series, int index) {
		final List<BoxplotTuple> boxplotTuples = _seriesMap.get(series);
		if (boxplotTuples != null) {
			return boxplotTuples.get(index).getMedian();
		}
		return null;
	}

	public Number getQ3(Comparable<?> series, int index) {
		final List<BoxplotTuple> boxplotTuples = _seriesMap.get(series);
		if (boxplotTuples != null) {
			return boxplotTuples.get(index).getQ3();
		}
		return null;
	}

	public Number getHigh(Comparable<?> series, int index) {
		final List<BoxplotTuple> boxplotTuples = _seriesMap.get(series);
		if (boxplotTuples != null) {
			return boxplotTuples.get(index).getHigh();
		}
		return null;
	}

	public void setValue(Comparable<?> series, Number low, Number q1, Number median, Number q3, Number high,
			int index) {
		removeValue0(series, index);
		int cIndex = addValue0(series, null, null, low, q1, median, q3, high, index);
		fireEvent(ChartDataEvent.CHANGED, series, null, _seriesList.indexOf(series), cIndex, new Number[]{low, q1, median, q3, high});
	}

	public void setValue(Comparable<?> series, Number x, Number low, Number q1, Number median, Number q3, Number high,
			int index) {
		removeValue0(series, index);
		int cIndex = addValue0(series, null, x, low, q1, median, q3, high, index);
		fireEvent(ChartDataEvent.CHANGED, series, (Comparable<?>) x, _seriesList.indexOf(series), cIndex, new Number[]{x, low, q1, median, q3, high});		
	}
	
	public void setValue(Comparable<?> series, String name, Number low, Number q1, Number median, Number q3,
			Number high, int index) {
		removeValue0(series, index);
		int cIndex = addValue0(series, name, null, low, q1, median, q3, high, index);
		fireEvent(ChartDataEvent.CHANGED, series, null, _seriesList.indexOf(series), cIndex, new Number[]{low, q1, median, q3, high});		
	}
	private int addValue0(Comparable<?> series, String name, Number x, Number low, Number q1, Number median, Number q3, Number high,
			int index) {
		List<BoxplotTuple> boxplotTuples = _seriesMap.get(series);
		if (boxplotTuples == null) {
			boxplotTuples = new ArrayList<BoxplotTuple>();
			_seriesMap.put(series, boxplotTuples);
			_seriesList.add(series);
		}
		int i = index;
		if (i >= 0) {
			if (name != null) {
				boxplotTuples.add(index, new BoxplotTuple(name, low, q1, median, q3, high));
			} else if (x != null) {
				boxplotTuples.add(index, new BoxplotTuple(x, low, q1, median, q3, high));
			} else {
				boxplotTuples.add(index, new BoxplotTuple(low, q1, median, q3, high));
			}
		} else {
			i = boxplotTuples.size();
			if (name != null) {
				boxplotTuples.add(new BoxplotTuple(name, low, q1, median, q3, high));
			} else if (x != null) {
				boxplotTuples.add(new BoxplotTuple(x, low, q1, median, q3, high));
			} else {
				boxplotTuples.add(new BoxplotTuple(low, q1, median, q3, high));
			}
		}
		return i;
	}


	public void addValue(Comparable<?> series, Number low, Number q1, Number median, Number q3, Number high) {
		addValue(series, low, q1, median, q3, high, -1);
	}

	public void addValue(Comparable<?> series, Number x, Number low, Number q1, Number median, Number q3, Number high) {
		addValue(series, x, low, q1, median, q3, high, -1);
	}

	public void addValue(Comparable<?> series, String name, Number low, Number q1, Number median, Number q3,
			Number high) {
		addValue(series, name, low, q1, median, q3, high, -1);
	}

	public void addValue(Comparable<?> series, Number low, Number q1, Number median, Number q3, Number high,
			int index) {
		int cIndex = addValue0(series, null, null, low, q1, median, q3, high, index);
		fireEvent(ChartDataEvent.ADDED, series, null, _seriesList.indexOf(series), cIndex, new Number[]{low, q1, median, q3, high});
	}

	public void addValue(Comparable<?> series, Number x, Number low, Number q1, Number median, Number q3, Number high,
			int index) {
		int cIndex = addValue0(series, null, x, low, q1, median, q3, high, index);
		fireEvent(ChartDataEvent.ADDED, series, (Comparable<?>) x, _seriesList.indexOf(series), cIndex, new Number[]{x, low, q1, median, q3, high});
	}

	public void addValue(Comparable<?> series, String name, Number low, Number q1, Number median, Number q3,
			Number high, int index) {
		int cIndex = addValue0(series, name, null, low, q1, median, q3, high, index);
		fireEvent(ChartDataEvent.ADDED, series, null, _seriesList.indexOf(series), cIndex, new Number[]{low, q1, median, q3, high});
	}

	public void removeSeries(Comparable<?> series) {
		_seriesMap.remove(series);
		
		final int index = _seriesList.indexOf(series);
		_seriesList.remove(series);
		fireEvent(ChartDataEvent.REMOVED, series, null, index, -1, null);
	}

	public void removeValue(Comparable<?> series, int index) {
		BoxplotTuple boxplotTuple = removeValue0(series, index);
		if (boxplotTuple != null) {
			fireEvent(ChartDataEvent.REMOVED, series, (Comparable<?>) boxplotTuple.getX(), _seriesList.indexOf(series), index, boxplotTuple.toNumbers());
		} else {
			fireEvent(ChartDataEvent.REMOVED, series, null, _seriesList.indexOf(series), index, null);
		}
	}
	
	private BoxplotTuple removeValue0(Comparable<?> series, int index) {
		List<BoxplotTuple> boxplotTuples = _seriesMap.get(series);
		if (boxplotTuples == null) {
			return null;
		}
		return boxplotTuples.remove(index);
	}

	public void clear() {
		_seriesMap.clear();
		_seriesList.clear();
		fireEvent(ChartDataEvent.REMOVED, null, null, -1, -1, null);
	}

	protected static class BoxplotTuple implements Serializable{
		private static final long serialVersionUID = -6426733466307214311L;
		private String _name;
		private Number _x;
		private Number _low;
		private Number _q1;
		private Number _median;
		private Number _q3;
		private Number _high;
		
		protected BoxplotTuple(Number low, Number q1, Number median, Number q3, Number high) {
			_low = low;
			_q1 = q1;
			_median = median;
			_q3 = q3;
			_high = high;
		}
		
		protected BoxplotTuple(Number x, Number low, Number q1, Number median, Number q3, Number high) {
			_x = x;
			_low = low;
			_q1 = q1;
			_median = median;
			_q3 = q3;
			_high = high;
		}
		
		protected BoxplotTuple(String name, Number low, Number q1, Number median, Number q3, Number high) {
			_name = name;
			_low = low;
			_q1 = q1;
			_median = median;
			_q3 = q3;
			_high = high;
		}
		
		public String getName() {
			return _name;
		}
		
		public Number getX() {
			return _x;
		}
		
		public Number getLow() {
			return _low;
		}
		
		public Number getQ1() {
			return _q1;
		}
		
		public Number getMedian() {
			return _median;
		}
		
		public Number getQ3() {
			return _q3;
		}
		
		public Number getHigh() {
			return _high;
		}
		
		public Number[] toNumbers() {
			return new Number[]{getLow(), getQ1(), getMedian(), getQ3(), getHigh()};
		}
	}
	
	public Object clone() {
		DefaultBoxPlotModel clone = (DefaultBoxPlotModel) super.clone();
		if (_seriesMap != null) {
			clone._seriesMap = new HashMap<Comparable<?>, List<BoxplotTuple>>(_seriesMap);
		}
		if (_seriesList != null) {
			clone._seriesList = new ArrayList<Comparable<?>>(_seriesList);
		}
		return clone;
	}
}
