/* DefaultFromToWeightModel.java

	Purpose:
		
	Description:
		
	History:
		Tue Dec 10 17:03:30 CST 2019, Created by rudyhuang

Copyright (C) 2019 Potix Corporation. All Rights Reserved.
*/
package org.zkoss.chart.model;

import java.util.ArrayList;
import java.util.List;

import org.zkoss.zul.event.ChartDataEvent;

/**
 * A default implementation of {@link FromToWeightModel}.
 * A FromToWeight model is an N series of (from, to, weight) data objects.
 *
 * @author rudyhuang
 * @since 7.2.1.0
 */
public class DefaultFromToWeightModel extends DefaultFromToModel implements FromToWeightModel {
	private static final long serialVersionUID = 20191210170415L;

	@Override
	public void setValue(Comparable<?> series, String from, String to, int index) {
		throw new UnsupportedOperationException("Use setValue(series, from, to, weight, index) instead!");
	}

	@Override
	public void addValue(Comparable<?> series, String from, String to) {
		throw new UnsupportedOperationException("Use addValue(series, from, to, weight) instead!");
	}

	@Override
	public void addValue(Comparable<?> series, String from, String to, int index) {
		throw new UnsupportedOperationException("Use addValue(series, from, to, weight, index) instead!");
	}

	public Number getWeight(Comparable<?> series, int index) {
		final List<FromToPair> tuples = _seriesMap.get(series);
		if (tuples != null) {
			return ((FromToWeightTuple) tuples.get(index)).getWeight();
		}
		return null;
	}

	public void addValue(Comparable<?> series, String from, String to, Number weight) {
		addValue(series, from, to, weight, -1);
	}

	public void addValue(Comparable<?> series, String from, String to, Number weight, int index) {
		int cIndex = addValue0(series, from, to, weight, index);
		fireEvent(ChartDataEvent.ADDED, series, from,
				_seriesList.indexOf(series), cIndex, new Object[] { to, weight });
	}

	public void setValue(Comparable<?> series, String from, String to, Number weight, int index) {
		removeValue0(series, index);
		int cIndex = addValue0(series, from, to, weight, index);
		fireEvent(ChartDataEvent.CHANGED, series, from,
				_seriesList.indexOf(series), cIndex, new Object[] { to, weight });
	}

	private int addValue0(Comparable<?> series, String from, String to, Number weight, int index) {
		List<FromToPair> tuples = _seriesMap.get(series);
		if (tuples == null) {
			tuples = new ArrayList<FromToPair>();
			_seriesMap.put(series, tuples);
			_seriesList.add(series);
		}
		int cIndex = index;
		if (index >= 0)
			tuples.add(index, new FromToWeightTuple(from, to, weight));
		else {
			cIndex = tuples.size();
			tuples.add(new FromToWeightTuple(from, to, weight));
		}
		return cIndex;
	}

	public void removeValue(Comparable<?> series, int index) {
		FromToWeightTuple old = removeValue0(series, index);
		fireEvent(ChartDataEvent.REMOVED, series, null,
					_seriesList.indexOf(series), old != null ? index : -1, null);
	}

	private FromToWeightTuple removeValue0(Comparable<?> series, int index) {
		List<FromToPair> tuples = _seriesMap.get(series);
		if (tuples == null) {
			return null;
		}
		return (FromToWeightTuple) tuples.remove(index);
	}

	//-- internal class --//
	private static class FromToWeightTuple extends FromToPair {
		private static final long serialVersionUID = 20191210170622L;
		private Number _weight;

		private FromToWeightTuple(String from, String to, Number weight) {
			super(from, to);
			_weight = weight;
		}

		public Number getWeight() {
			return _weight;
		}
	}
}
