/* DumbbellPlotOptions.java

	Purpose:
		
	Description:
		
	History:
		4:59 PM 2022/9/8, Created by jumperchen

Copyright (C) 2022 Potix Corporation. All Rights Reserved.
*/
package org.zkoss.chart.plotOptions;

import org.zkoss.chart.Color;
import org.zkoss.chart.LinearGradient;
import org.zkoss.chart.PlotAttribute;
import org.zkoss.chart.RadialGradient;
import org.zkoss.chart.util.DynamicalAttribute;

/**
 * An options for dumbbell plot.
 * <p>
 * All the options in this class support {@link DynamicalAttribute}.
 * @author jumperchen
 * @since 10.2.1.0
 */
public class DumbbellPlotOptions extends SeriesPlotOptions {
	private enum Attrs implements PlotAttribute, DynamicalAttribute {
		connectorColor,
		connectorWidth,
		lineColor,
		lowColor,
		negativeFillColor,
		pointPadding,
	}

	/**
	 * Returns the color of the line that connects the dumbbell point's values.
	 * By default it is the series' color.
	 *
	 * <p>Defaults to null.
	 */
	public String getConnectorColor() {
		return getAttr(Attrs.connectorColor, null).asString();
	}

	/**
	 * Sets the color of the line that connects the dumbbell point's values.
	 * By default it is the series' color.
	 *
	 * <p>Defaults to null.
	 */
	public void setConnectorColor(String connectorColor) {
		setAttr(Attrs.connectorColor, connectorColor);
	}

	/**
	 * Returns the pixel width of the line that connects the dumbbell point's values.
	 * <p>Default: 1
	 */
	public Number getConnectorWidth() {
		return getAttr(Attrs.connectorWidth, 1).asNumber();
	}

	/**
	 * Sets the pixel width of the line that connects the dumbbell point's values.
	 * <p>Default: 1
	 */
	public void setConnectorWidth(Number connectorWidth) {
		setAttr(Attrs.connectorWidth, connectorWidth);
	}

	/**
	 * Returns a separate color for the graph line. By default the line takes
	 * the <code>color</code> of the series, but the lineColor setting allows
	 * setting a separate color for the line without altering the
	 * <code>fillColor</code>.
	 * <p>
	 * Default: null.
	 */
	public Color getLineColor() {
		return (Color) getAttr(Attrs.lineColor, null).asValue();
	}

	/**
	 * Sets a separate color for the graph line. By default the line takes the
	 * <code>color</code> of the series, but the lineColor setting allows
	 * setting a separate color for the line without altering the
	 * <code>fillColor</code>.
	 */
	public void setLineColor(Color color) {
		setAttr(Attrs.lineColor, color);
	}

	/**
	 * Sets a separate color for the graph line. By default the line takes the
	 * <code>color</code> of the series, but the lineColor setting allows
	 * setting a separate color for the line without altering the
	 * <code>fillColor</code>.
	 */
	public void setLineColor(String color) {
		setLineColor(new Color(color));
	}

	/**
	 * Sets a separate color for the graph line. By default the line takes the
	 * <code>color</code> of the series, but the lineColor setting allows
	 * setting a separate color for the line without altering the
	 * <code>fillColor</code>.
	 */
	public void setLineColor(LinearGradient color) {
		setLineColor(new Color(color));
	}

	/**
	 * Sets a separate color for the graph line. By default the line takes the
	 * <code>color</code> of the series, but the lineColor setting allows
	 * setting a separate color for the line without altering the
	 * <code>fillColor</code>.
	 */
	public void setLineColor(RadialGradient color) {
		setLineColor(new Color(color));
	}

	/**
	 * Returns color of the start markers in a dumbbell graph.
	 * <p>
	 * Default: {@code "#333333"}.
	 */
	public Color getLowColor() {
		return (Color) getAttr(Attrs.lowColor, new Color("#333333")).asValue();
	}

	/**
	 * Sets color of the start markers in a dumbbell graph.
	 * <p>
	 * Default: {@code "#333333"}.
	 */
	public void setLowColor(Color color) {
		setAttr(Attrs.lowColor, color);
	}

	/**
	 * Sets color of the start markers in a dumbbell graph.
	 * <p>
	 * Default: {@code "#333333"}.
	 */
	public void setLowColor(String color) {
		setLowColor(new Color(color));
	}

	/**
	 * Sets color of the start markers in a dumbbell graph.
	 * <p>
	 * Default: {@code "#333333"}.
	 */
	public void setLowColor(LinearGradient color) {
		setLowColor(new Color(color));
	}

	/**
	 * Sets color of the start markers in a dumbbell graph.
	 * <p>
	 * Default: {@code "#333333"}.
	 */
	public void setLowColor(RadialGradient color) {
		setLowColor(new Color(color));
	}


	/**
	 * Returns a separate color for the negative part of the area.
	 * <p>
	 * Default: null.
	 */
	public Color getNegativeFillColor() {
		return (Color) getAttr(Attrs.negativeFillColor, null).asValue();
	}

	/**
	 * Sets a separate color for the negative part of the area.
	 * <p>
	 * Default: null.
	 */
	public void setNegativeFillColor(Color color) {
		setAttr(Attrs.negativeFillColor, color);
	}

	/**
	 * Sets a separate color for the negative part of the area.
	 * <p>
	 * Default: null.
	 */
	public void setNegativeFillColor(String color) {
		setNegativeFillColor(new Color(color));
	}

	/**
	 * Sets a separate color for the negative part of the area.
	 * <p>
	 * Default: null.
	 */
	public void setNegativeFillColor(LinearGradient color) {
		setNegativeFillColor(new Color(color));
	}

	/**
	 * Sets a separate color for the negative part of the area.
	 * <p>
	 * Default: null.
	 */
	public void setNegativeFillColor(RadialGradient color) {
		setNegativeFillColor(new Color(color));
	}

	/**
	 * Returns the padding between each column or bar, in x axis units.
	 * Default: 0.1
	 */
	public Number getPointPadding() {
		return getAttr(Attrs.pointPadding, 0.1).asNumber();
	}

	/**
	 * Sets the padding between each column or bar, in x axis units.
	 */
	public void setPointPadding(Number pointPadding) {
		setAttr(Attrs.pointPadding, pointPadding, 0.1);
	}
}
