/* StreamGraphPlotOptions.java

	Purpose:
		
	Description:
		
	History:
		Fri Nov 29 11:23:13 CST 2019, Created by rudyhuang

Copyright (C) 2019 Potix Corporation. All Rights Reserved.
*/
package org.zkoss.chart.plotOptions;

import org.zkoss.chart.Color;
import org.zkoss.chart.LinearGradient;
import org.zkoss.chart.PlotAttribute;
import org.zkoss.chart.RadialGradient;
import org.zkoss.chart.util.DynamicalAttribute;

/**
 * An option for streamgraph plot
 * <p>
 * All the options in this class support {@link DynamicalAttribute}.
 *
 * @author rudyhuang
 * @since 7.2.1.0
 */
public class StreamGraphPlotOptions extends SeriesPlotOptions {
	private enum Attrs implements PlotAttribute, DynamicalAttribute {
		fillColor,
		fillOpacity,
		lineColor,
		negativeFillColor,
		trackByArea
	}

	/**
	 * Returns fill color or gradient for the area. When <code>null</code>, the
	 * series' <code>color</code> is used with the series'
	 * <code>fillOpacity</code>.
	 * <p>
	 * Default: null.
	 * @since 10.2.1.0
	 */
	public Color getFillColor() {
		return (Color) getAttr(Attrs.fillColor, null).asValue();
	}

	/**
	 * Sets fill color or gradient for the area. When <code>null</code>, the
	 * series' <code>color</code> is used with the series'
	 * <code>fillOpacity</code>.
	 * @since 10.2.1.0
	 */
	public void setFillColor(Color color) {
		setAttr(Attrs.fillColor, color);
	}

	/**
	 * Sets fill color or gradient for the area. When <code>null</code>, the
	 * series' <code>color</code> is used with the series'
	 * <code>fillOpacity</code>.
	 * @since 10.2.1.0
	 */
	public void setFillColor(String color) {
		setFillColor(new Color(color));
	}

	/**
	 * Sets fill color or gradient for the area. When <code>null</code>, the
	 * series' <code>color</code> is used with the series'
	 * <code>fillOpacity</code>.
	 * @since 10.2.1.0
	 */
	public void setFillColor(LinearGradient color) {
		setFillColor(new Color(color));
	}

	/**
	 * Sets fill color or gradient for the area. When <code>null</code>, the
	 * series' <code>color</code> is used with the series'
	 * <code>fillOpacity</code>.
	 * @since 10.2.1.0
	 */
	public void setFillColor(RadialGradient color) {
		setFillColor(new Color(color));
	}

	/**
	 * Returns fill opacity
	 * <p>
	 * Default: 1.
	 * @since 10.2.1.0
	 */
	public Number getFillOpacity() {
		return getAttr(Attrs.fillOpacity, 1).asNumber();
	}

	/**
	 * Sets fill opacity
	 * @since 10.2.1.0
	 */
	public void setFillOpacity(Number fillOpacity) {
		setAttr(Attrs.fillOpacity, fillOpacity);
	}

	/**
	 * Returns a separate color for the graph line. By default the line takes
	 * the <code>color</code> of the series, but the lineColor setting allows
	 * setting a separate color for the line without altering the
	 * <code>fillColor</code>.
	 * <p>
	 * Default: null.
	 * @since 10.2.1.0
	 */
	public Color getLineColor() {
		return (Color) getAttr(Attrs.lineColor, null).asValue();
	}

	/**
	 * Sets a separate color for the graph line. By default the line takes the
	 * <code>color</code> of the series, but the lineColor setting allows
	 * setting a separate color for the line without altering the
	 * <code>fillColor</code>.
	 * @since 10.2.1.0
	 */
	public void setLineColor(Color color) {
		setAttr(Attrs.lineColor, color);
	}

	/**
	 * Sets a separate color for the graph line. By default the line takes the
	 * <code>color</code> of the series, but the lineColor setting allows
	 * setting a separate color for the line without altering the
	 * <code>fillColor</code>.
	 * @since 10.2.1.0
	 */
	public void setLineColor(String color) {
		setLineColor(new Color(color));
	}

	/**
	 * Sets a separate color for the graph line. By default the line takes the
	 * <code>color</code> of the series, but the lineColor setting allows
	 * setting a separate color for the line without altering the
	 * <code>fillColor</code>.
	 * @since 10.2.1.0
	 */
	public void setLineColor(LinearGradient color) {
		setLineColor(new Color(color));
	}

	/**
	 * Sets a separate color for the graph line. By default the line takes the
	 * <code>color</code> of the series, but the lineColor setting allows
	 * setting a separate color for the line without altering the
	 * <code>fillColor</code>.
	 * @since 10.2.1.0
	 */
	public void setLineColor(RadialGradient color) {
		setLineColor(new Color(color));
	}

	/**
	 * Returns a separate color for the negative part of the area.
	 * <p>
	 * Default: null.
	 * @since 10.2.1.0
	 */
	public Color getNegativeFillColor() {
		return (Color) getAttr(Attrs.negativeFillColor, null).asValue();
	}

	/**
	 * Sets a separate color for the negative part of the area.
	 * <p>
	 * Default: null.
	 * @since 10.2.1.0
	 */
	public void setNegativeFillColor(Color color) {
		setAttr(Attrs.negativeFillColor, color);
	}

	/**
	 * Sets a separate color for the negative part of the area.
	 * <p>
	 * Default: null.
	 * @since 10.2.1.0
	 */
	public void setNegativeFillColor(String color) {
		setNegativeFillColor(new Color(color));
	}

	/**
	 * Sets a separate color for the negative part of the area.
	 * <p>
	 * Default: null.
	 * @since 10.2.1.0
	 */
	public void setNegativeFillColor(LinearGradient color) {
		setNegativeFillColor(new Color(color));
	}

	/**
	 * Sets a separate color for the negative part of the area.
	 * <p>
	 * Default: null.
	 * @since 10.2.1.0
	 */
	public void setNegativeFillColor(RadialGradient color) {
		setNegativeFillColor(new Color(color));
	}

	/**
	 * Returns whether the whole area or just the line should respond to
	 * mouseover tooltips and other mouse or touch events.
	 * <p>
	 * Default: false.
	 * @since 10.2.1.0
	 */
	public boolean getTrackByArea() {
		return getAttr(Attrs.trackByArea, false).asBoolean();
	}

	/**
	 * Sets whether the whole area or just the line should respond to mouseover
	 * tooltips and other mouse or touch events.
	 * @since 10.2.1.0
	 */
	public void setTrackByArea(boolean trackByArea) {
		setAttr(Attrs.trackByArea, trackByArea);
	}
}
